#version 330 core
#define PI 3.1415926535897932384626433832795
out vec4 FragColor;

in vec3 Normal;  
in vec3 FragPos;  
in vec2 TexCoords;

uniform sampler2D texture1;
uniform vec3 viewPos;
uniform vec3 lightPos;
uniform float time;

uniform bool textureEnabled = true;

void main()
{    
	float r = sin(time);
	float g = sin(time + (PI/2));
	float b = sin(time + PI);

	vec3 lightColor = (vec3(r,g,b) + vec3(2)) / 2;

	// ambient
    float ambientStrength = 0.4;
    vec3 ambient = ambientStrength * lightColor;
  	
    // diffuse 
    vec3 norm = normalize(Normal);
    vec3 lightDir = normalize(lightPos - FragPos);
    float diff = max(dot(norm, lightDir), 0.0);
    vec3 diffuse = diff * lightColor;

	// specular
	float specularStrength = 0.5;
	vec3 viewDir = normalize(viewPos - FragPos);
	vec3 reflectDir = reflect(-lightDir, norm); 
	
	float spec = pow(max(dot(viewDir, reflectDir), 0.0), 32);
	vec3 specular = specularStrength * spec * lightColor; 
	
	// resultado
	vec4 color;
	if (textureEnabled)
		color = texture(texture1, TexCoords);
	else
		color = vec4(0.5f, 0.5f, 0.5f, 1);
    FragColor = vec4((ambient + diffuse + specular),1) * color;
}