
layout(early_fragment_tests) in;

out vec4 fcolor;

// Inputs from the VS
#ifndef INTERPOLATE
flat in vec3 WPos;
flat in vec3 ShadowPos;
#else
in vec3 WPos;
in vec3 ShadowPos;
#endif

// Uniforms
uniform vec3 CameraPos;
uniform vec3 ObjColor;
uniform sampler2D DiffTexture;
uniform sampler2D NormalTexture;
uniform samplerCube EnvMap;
uniform vec3 SunDir;
uniform float CurrentDisp;

//uniform sampler2DShadow ShadowBuffer;
uniform sampler2D ShadowBuffer;

uniform vec3 DiskPos;
uniform vec3 PrevDiskPos;
uniform vec3 SunColor;

#define PI 3.14159f
// From UE4
vec2 Hammersley( uint Index, uint NumSamples,vec2 px )
{
	uint x = uint(px.x);
	uint y = uint(px.y);

	x = x * 1664525u + 1013904223u;
	y = y * 1664525u + 1013904223u;
	
	uint z = uint(1024*gl_FragCoord.z); 
	x += y*z;
	y += z*x;
	z += x*y;
	x += y*z;
	y += z*x;

	x = x >> 16u;
	y = y >> 16u;

	float E1 = fract( float(Index) / float(NumSamples) + float( x & 0xffff ) / (1<<16) );
	float E2 = float( bitfieldReverse(Index) ^ y ) * 2.3283064365386963e-10;
	return vec2( E1, E2 );
}

vec2 rand(vec2 px,float i,float n)
{	
	return Hammersley(uint(i),uint(n),px);
}

float SampleShadow(float BaseBias,float NoL, const float SamplesN,const float Radius)
{
	float ret = 0;
	NoL = clamp(NoL,0.01f,1);
	float a = 0.25f;
	float bias = BaseBias*((1 - NoL)/(a*NoL) + 1); // Approximates tan(acos(NoL))

	for(float i = 0;i < SamplesN;i++)
	{
		float z0 = ShadowPos.z;
		float zs = texture( ShadowBuffer, ShadowPos.xy + Radius*rand(gl_FragCoord.xy,i,SamplesN)).r;
		
		// Based on the integral of zs <= z0 - b*r ? 0.0f : 1.0f, with respect to r in the [0.1,1] range
		// Ajusted with a power later it softer
		ret += pow(clamp((bias - z0 + zs)/bias,0.0f,1.0f),0.7);
	}

	return clamp(ret * (1.0f / SamplesN),0.0f,1.0f);
}
void main()
{
	fcolor = vec4(0);

	float tile_size = 0.4f;
	vec2 UV = WPos.xy*tile_size;

	float roughness = 0.1;
	float alpha = roughness*roughness;

	vec3 n = texture(NormalTexture,UV).rgb*2-1;
	n = normalize(n);
	float NoL = clamp( dot( n, SunDir ) ,0,1);

	vec3 v = normalize(CameraPos - WPos);
	vec3 h = normalize(v + SunDir);
	vec3 r = reflect(v,n);

	vec3 albedo = texture(DiffTexture,UV).rgb;

	vec3 ambient = textureLod(EnvMap,n,9).rgb; // Using a coarse mip as a preintegrated ambient

	vec3 diff = vec3(NoL);
	// Using simple phong to make it faster
	vec3 spec = vec3(pow(dot(n,h),1.0 / alpha));

	float ka = 0.75f;
	float kd = 0.75f;
	float ks = 0.5f;
	float kl = 1.75f * SampleShadow(0.0003f,NoL,4,0.0004f); // light intensity
	fcolor.rgb = ObjColor*albedo*(ka*ambient + SunColor*kl*(kd*diff + ks*spec));

	// Make lower parts of the blade darker
	fcolor.rgb *= pow(max(0.5,min(1.0f,2.85*CurrentDisp)),2.0f);
	
	float disp = 0.3333f*(albedo.x + albedo.y + albedo.z);

	// Check if the grass needs to be squashed because something passed over it
	/*
	{
		// Check for the disk
		const float r = 2.1f; // TODO : Expose this
		if(dot(WPos-PrevDiskPos,WPos-PrevDiskPos) < r*r || dot(WPos-DiskPos,WPos-DiskPos) < r*r )
		{
			disp *= 0.1f;//clamp(distance(WPos, DiskPos)/r,0,1);
			//fcolor.rgb *= 1.5f;
		}
		
		// TODO : COMPUTE THIS ON THE CPU!!!!
		vec2 center = 0.5f*vec2(DiskPos + PrevDiskPos);
		vec2 cpos = vec2(DiskPos) - center;
		vec2 ppos = vec2(PrevDiskPos) - center;

		vec2 dir = normalize(cpos-ppos);
		float angle = sign(dir.y)*acos(dot(dir,vec2(1,0)));
		mat2 rot = {{cos(angle),-sin(angle)},{sin(angle),cos(angle)}};

		vec2 pos = (vec2(WPos) - center) * rot;
		cpos *= rot;
		ppos *= rot;

		vec2 min_pos = min(cpos,ppos);
		vec2 max_pos = max(cpos,ppos);
		if(pos.x > -r && pos.x < r && pos.y > min_pos.y && pos.y < max_pos.y)
		{
			disp *= 0.1f;
		}
	}*/

	float rand_offset = length((cos(0.25*WPos) + sin(0.5*WPos.yxz)));
	rand_offset *= 0.2f*clamp(dot(v,n),0,1);//0.3f;

	float cutoff_h = (WPos.z + 0.575) + 0.2*clamp(dot(v,n),0,1) + 0.1;
	//float cutoff_h = CurrentDisp + rand_offset;
	float cutoff_w = 50.5f;
	disp -= cutoff_h;

	if(CurrentDisp <= 0.01f)
		fcolor.a = 1;
	else
		fcolor.a = (cutoff_w*sign(disp)*abs(disp) / (cutoff_w*abs(disp) + 1))*0.5f + 0.5f;
	
	/*fcolor.a = 1;
	fcolor.rgb = 0.5*albedo + 0.5*textureLod(EnvMap,-r,0).rgb;*/
}