

// Define input layout
layout(location = 0) in vec3 InPos;
layout(location = 1) in vec3 InNormal;
layout(location = 2) in vec2 InUV;

// Define uniforms
uniform mat4 World;
uniform mat4 View;
uniform mat4 Proj;
uniform mat4 PVW;
uniform mat4 ShadowPVW;

// Define output
#ifndef INTERPOLATE
flat out vec3 WPos;
flat out vec3 WNormal;
flat out vec2 UV;
flat out vec3 ShadowPos;
#else
out vec3 WPos;
out vec3 WNormal;
out vec2 UV;
out vec3 ShadowPos;
#endif


void main()
{
	gl_Position = PVW*vec4(InPos,1.0f);

	// Convert to mat3 to get only the rotation part
	WNormal = normalize((mat3(World)*InNormal).xyz);

	UV = InUV;
	UV.y = 1 - UV.y;
	WPos = (World*vec4(InPos,1.0f)).xyz;

	vec4 sh = ShadowPVW*vec4(InPos,1.0f);
	sh.xyz /= sh.w;
	sh.xyz = sh.xyz*0.5 + 0.5; // z is also in [-1,1] range in gl

	ShadowPos = sh.xyz;
}